"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearAllTempDatabases = exports.clearTempDatabasesOlderThan = exports.clearTempDatabase = exports.createTempDatabase = exports.TEMPORARY_DB_PREFIX = void 0;
const uuid = __importStar(require("uuid"));
const neo4j_driver_1 = __importDefault(require("neo4j-driver"));
exports.TEMPORARY_DB_PREFIX = 'tempneogmadb';
const getCurrentTimestamp = () => {
    return Math.floor(new Date().getTime() / 1000);
};
const filterConsoleDatabasesFromResult = (result) => {
    return result.records.filter((record) => record.get('name').indexOf(exports.TEMPORARY_DB_PREFIX) === 0);
};
const deleteDatabaseUserAndRole = async (session, database) => {
    try {
        await session.run(`STOP DATABASE ${database};`);
    }
    catch (error) {
        console.error(error);
    }
    try {
        await session.run(`DROP DATABASE ${database};`);
    }
    catch (error) {
        console.error(error);
    }
    try {
        await session.run(`DROP USER ${database};`);
    }
    catch (error) {
        console.error(error);
    }
    try {
        await session.run(`DROP ROLE ${database};`);
    }
    catch (error) {
        console.error(error);
    }
};
const createTempDatabase = async (driver) => {
    const sessionId = uuid.v4().replace(/-/g, '');
    const currentTimestamp = getCurrentTimestamp();
    const database = `${exports.TEMPORARY_DB_PREFIX}${sessionId}${currentTimestamp}`;
    const session = driver.session({
        database: 'system',
        defaultAccessMode: neo4j_driver_1.default.session.WRITE,
    });
    try {
        await session.run(`CREATE DATABASE ${database} WAIT;`);
        await session.run(`CREATE USER ${database} SET PASSWORD '${database}' SET PASSWORD CHANGE NOT REQUIRED;`);
        await session.run(`CREATE ROLE ${database};`);
        await session.run(`GRANT ROLE ${database} TO ${database};`);
        await session.run(`GRANT ALL ON DATABASE ${database} TO ${database};`);
        await session.run(`GRANT ACCESS ON DATABASE ${database} TO ${database};`);
        await session.run(`GRANT READ {*} ON GRAPH ${database} TO ${database};`);
        await session.run(`GRANT TRAVERSE ON GRAPH ${database} TO ${database};`);
        await session.run(`GRANT WRITE ON GRAPH ${database} TO ${database};`);
    }
    catch (error) {
        console.error(error);
    }
    finally {
        await session.close();
    }
    return database;
};
exports.createTempDatabase = createTempDatabase;
const clearTempDatabase = async (driver, database) => {
    const session = driver.session({ database: 'system' });
    try {
        await deleteDatabaseUserAndRole(session, database);
    }
    catch (error) {
        console.error(error);
    }
    finally {
        await session.close();
    }
};
exports.clearTempDatabase = clearTempDatabase;
const clearTempDatabasesOlderThan = async (driver, seconds) => {
    const session = driver.session({ database: 'system' });
    const result = await session.run('SHOW DATABASES');
    const shouldExpireAt = getCurrentTimestamp() - seconds;
    try {
        const records = filterConsoleDatabasesFromResult(result);
        console.log('Databases found: ' + records.length);
        for (const record of records) {
            const database = record.get('name'); //tempneogmadb56e7794ad165454282ee0a7c32a5e3eb1705341040
            const dbTimestamp = parseInt(database.slice(44), 10);
            const isExpired = dbTimestamp <= shouldExpireAt;
            if (isExpired) {
                await deleteDatabaseUserAndRole(session, database);
            }
            else {
            }
        }
    }
    catch (error) {
        console.error(error);
    }
    finally {
        await session.close();
    }
};
exports.clearTempDatabasesOlderThan = clearTempDatabasesOlderThan;
const clearAllTempDatabases = async (driver) => {
    const session = driver.session({ database: 'system' });
    const result = await session.run('SHOW DATABASES');
    try {
        const records = filterConsoleDatabasesFromResult(result);
        for (const record of records) {
            const database = record.get('name');
            await deleteDatabaseUserAndRole(session, database);
        }
    }
    catch (error) {
        console.error(error);
    }
    finally {
        await session.close();
    }
};
exports.clearAllTempDatabases = clearAllTempDatabases;
//# sourceMappingURL=temp.js.map